Module dictionary

Module dictionary 

Available on crate feature dict_builder only.
Code for creating a separate content dictionary.

Effective dictionaries are up to 1% the size of the complete training body, and are trained on many examples of the original data.

Implemented following the paper “Effective construction of Relative Lempel-Ziv Dictionaries”, by Kewen Liao, Matthias Petri, Alistair Moffat, and Anthony Wirth

Structs

FastCoverOptions
Tuning knobs for pure-Rust FastCOVER training.
FastCoverParams
FastCoverTuned
FinalizeOptions

Constants

DEFAULT_D_CANDIDATES
DEFAULT_F_CANDIDATES
DEFAULT_K_CANDIDATES

Functions

create_fastcover_dict_from_source
Train and finalize a FastCOVER dictionary in pure Rust.
create_fastcover_raw_dict_from_source
Train a raw FastCOVER dictionary from a source stream.
create_raw_dict_from_dir
Creates a “raw content” dictionary, training off of every file in this directory and all sub-directories.
create_raw_dict_from_source
Read from source to create a “raw content” dictionary of dict_size. The completed dictionary is written to output.
finalize_raw_dict
Finalize raw dictionary content into a full zstd dictionary binary (magic + dict_id + entropy tables + offset history + content).
train_fastcover_raw_from_slice
Train a raw FastCOVER dictionary directly from an in-memory sample.