Module decoding

Module decoding 

Structures and utilities used for decoding zstd formatted data

Modules

errors
Errors that might occur while decoding zstd formatted data

Structs

Dictionary
Zstandard includes support for “raw content” dictionaries, that store bytes optionally used during sequence execution.
FrameDecoder
Low level Zstandard decoder that can be used to decompress frames with fine control over when and how many bytes are decoded.
StreamingDecoder
High level Zstandard frame decoder that can be used to decompress a given Zstandard frame.

Enums

BlockDecodingStrategy