Structures and utilities used for decoding zstd formatted data.
Use DictionaryHandle for pre-parsed dictionary reuse across repeated decode paths.
Modules
- errors
- Errors that might occur while decoding zstd formatted data
Structs
- Dictionary
- Zstandard includes support for “raw content” dictionaries, that store bytes optionally used during sequence execution.
- Dictionary
Handle - Shared pre-parsed dictionary handle for repeated decoding.
- Frame
Decoder - Low level Zstandard decoder that can be used to decompress frames with fine control over when and how many bytes are decoded.
- Streaming
Decoder - High level Zstandard frame decoder that can be used to decompress a given Zstandard frame.