pub fn frame_header_size(src: &[u8]) -> Result<usize, ReadFrameHeaderError>Expand description
Length in bytes of the frame header at the start of src, including the
4-byte magic number (the offset at which the first block begins). Backs the
C ZSTD_frameHeaderSize.
Errors
ReadFrameHeaderError when the header is
too short, has a bad magic number, or is a skippable frame.