Available on crate feature
dict_builder only.Expand description
Code for creating a separate content dictionary.
Effective dictionaries are up to 1% the size of the complete training body, and are trained on many examples of the original data.
Implemented following the paper “Effective construction of Relative Lempel-Ziv Dictionaries”, by Kewen Liao, Matthias Petri, Alistair Moffat, and Anthony Wirth
Structs
- Fast
Cover Options - Tuning knobs for pure-Rust FastCOVER training.
- Fast
Cover Params - Fast
Cover Tuned - Finalize
Options
Constants
Functions
- create_
fastcover_ dict_ from_ source - Train and finalize a FastCOVER dictionary in pure Rust.
- create_
fastcover_ raw_ dict_ from_ source - Train a raw FastCOVER dictionary from a source stream.
- create_
raw_ dict_ from_ dir - Creates a “raw content” dictionary, training off of every file in this directory and all sub-directories.
- create_
raw_ dict_ from_ source - Read from
sourceto create a “raw content” dictionary ofdict_size. The completed dictionary is written tooutput. - finalize_
raw_ dict - Finalize raw dictionary content into a full zstd dictionary binary
(
magic + dict_id + entropy tables + offset history + content). - train_
fastcover_ raw_ from_ slice - Train a raw FastCOVER dictionary directly from an in-memory sample.