pub enum CompressionLevel {
Uncompressed,
Fastest,
Default,
Better,
Best,
}Expand description
The compression mode used impacts the speed of compression, and resulting compression ratios. Faster compression will result in worse compression ratios, and vice versa.
Variants
Uncompressed
This level does not compress the data at all, and simply wraps it in a Zstandard frame.
Fastest
This level is roughly equivalent to Zstd compression level 1
Default
This level uses the crate’s dedicated dfast-style matcher to
target a better speed/ratio tradeoff than CompressionLevel::Fastest.
It represents this crate’s “default” compression setting and may evolve in future versions as the implementation moves closer to reference zstd level 3 behavior.
Better
This level is roughly equivalent to Zstd level 7.
UNIMPLEMENTED
Best
This level is roughly equivalent to Zstd level 11.
UNIMPLEMENTED